@@ -5,27 +5,13 @@
 * ABSTRACT: help system
 */
 
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
-#include <stddef.h>
-#include <stdlib.h>
-#include <time.h>
-
-
-
-
-#include <kernel/mod2.h>
-
-#include <omalloc/omalloc.h>
-#include <misc/mylimits.h>
-
-#include <resources/feResource.h>
-#include <reporter/reporter.h>
-
-#include <resources/omFindExec.h>
-
-#include <reporter/si_signals.h>
+#include "kernel/mod2.h"
+#include "omalloc/omalloc.h"
+#include "misc/mylimits.h"
+#include "resources/feResource.h"
+#include "reporter/reporter.h"
+#include "resources/omFindExec.h"
+#include "reporter/si_signals.h"
 
 #include "ipid.h"
 #include "ipshell.h"
@@ -82,8 +68,8 @@ static BOOLEAN heGenInit(int,int);    static void heGenHelp(heEntry hentry,int);
 static BOOLEAN heDummyInit(int,int);   static void heDummyHelp(heEntry hentry,int);
 static BOOLEAN heEmacsInit(int,int);   static void heEmacsHelp(heEntry hentry,int);
 
-static heBrowser heCurrentHelpBrowser = NULL;
-static int heCurrentHelpBrowserIndex= -1;
+STATIC_VAR heBrowser heCurrentHelpBrowser = NULL;
+STATIC_VAR int heCurrentHelpBrowserIndex= -1;
 
 
 /*****************************************************************
@@ -93,7 +79,7 @@ static int heCurrentHelpBrowserIndex= -1;
  *****************************************************************/
 // order is important -- first possible help is chosen
 // moved to LIB/help.cnf
-static heBrowser_s *heHelpBrowsers=NULL;
+STATIC_VAR heBrowser_s *heHelpBrowsers=NULL;
 
 /*****************************************************************
  *
@@ -172,7 +158,7 @@ void feHelp(char *str)
       Warn("No unique help for '%s'", str);
     else
       Warn("No help for topic '%s'", str);
-    Warn("Try one of");
+    WarnS("Try one of");
     char *matches=StringEndS();
     PrintS(matches);
     omFree(matches);
@@ -712,7 +698,7 @@ static BOOLEAN heOnlineHelp(char* s)
        ||
        ((fp=feFopen(str,"rb", libnamebuf))!=NULL)))
   {
-    extern FILE *yylpin;
+    EXTERN_VAR FILE *yylpin;
     lib_style_types lib_style; // = OLD_LIBSTYLE;
 
     yylpin = fp;
@@ -777,7 +763,7 @@ static long heKeyChksum(char* key)
  *
  *****************************************************************/
 
-static BOOLEAN feHelpCalled = FALSE;
+STATIC_VAR BOOLEAN feHelpCalled = FALSE;
 
 static void heBrowserHelp(heEntry hentry)
 {
@@ -797,7 +783,7 @@ static void heBrowserHelp(heEntry hentry)
     //{
     //  Warn("Using URL '%s'.", feResource('u', 0));
     //}
-    Warn("Use 'system(\"--browser\", <browser>);' to change browser,");
+    WarnS("Use 'system(\"--browser\", <browser>);' to change browser,");
     StringSetS("where <browser> can be: ");
     int i = 0;
     i = 0;
@@ -910,10 +896,16 @@ static void heGenHelp(heEntry hentry, int br)
                    #ifdef HAVE_VSNPRINTF
                    {
                      if (*p=='H')
+                     #ifdef SINGULAR_4_2
+                       snprintf(temp,256,"%s/%d-%d/%s", htmldir,
+                                  SINGULAR_VERSION/1000,
+                                 (SINGULAR_VERSION % 1000)/100,
+                     #else
                        snprintf(temp,256,"%s/%d-%d-%d/%s", htmldir,
                                   SINGULAR_VERSION/1000,
                                  (SINGULAR_VERSION % 1000)/100,
                                  (SINGULAR_VERSION % 100)/10,
+                     #endif
                        hentry->url);
                      else
                        snprintf(temp,256,"%s/%s", htmldir, hentry->url);
@@ -1032,15 +1024,15 @@ static void heEmacsHelp(heEntry hentry, int /*br*/)
   WarnS("Your help command could not be executed. Use");
   Warn("C-h C-s %s",
        (hentry != NULL && *(hentry->node) != '\0' ? hentry->node : "Top"));
-  Warn("to enter the Singular online help. For general");
-  Warn("information on Singular running under Emacs, type C-h m.");
+  WarnS("to enter the Singular online help. For general");
+  WarnS("information on Singular running under Emacs, type C-h m.");
 }
 static int singular_manual(char *str, BOOLEAN isIndexEntry);
 static void heBuiltinHelp(heEntry hentry, int /*br*/)
 {
   char* node = omStrDup(hentry != NULL && *(hentry->key) != '\0' ?
                        hentry->key : "Top");
-  singular_manual(node,(hentry != NULL) && (hentry->url!=NULL));
+  singular_manual(node,(hentry != NULL) && *(hentry->url)!='\0');
   omFree(node);
 }
 
@@ -1055,7 +1047,6 @@ static void heBuiltinHelp(heEntry hentry, int /*br*/)
 #define HELP_NOT_FOUND 2
 #define BUF_LEN        256
 #define IDX_LEN        256
-#define MAX_LINES      21
 
 static inline char tolow(char p)
 {
@@ -1078,7 +1069,7 @@ static int show(unsigned long offset, char *close)
         && (buffer[0] != FIN_INDEX))
   {
     printf("%s", buffer);
-    if(lines++> MAX_LINES)
+    if(lines++==pagelength)
     {
       printf("\n Press <RETURN> to continue or x to exit help.\n");
       fflush(stdout);
@@ -1091,7 +1082,7 @@ static int show(unsigned long offset, char *close)
       lines=0;
     }
   }
-  if(*close!='x')
+  if((*close!='x')&&(pagelength>0))
   {
     printf("\nEnd of part. Press <RETURN> to continue or x to exit help.\n");
     fflush(stdout);
