@@ -170,62 +170,54 @@ static void jjMINPOLY_red(idhdl h)
        Werror("type %d too complex...set minpoly before",IDTYP(h)); break;
   }
 }
-ring jjSetMinpoly(ring r, number a)
+// change the coeff cf=K[x] (of type n_transExt) to K[x]/a
+// return NULL in error case
+coeffs jjSetMinpoly(coeffs cf, number a)
 {
-  if ( !nCoeff_is_transExt(r->cf) )
+  if ( !nCoeff_is_transExt(cf) )
   {
-    if(!nCoeff_is_algExt(r->cf) )
+    if(!nCoeff_is_algExt(cf) )
     {
       WerrorS("cannot set minpoly for these coeffients");
       return NULL;
     }
   }
-  if (rVar(r->cf->extRing)!=1)
+  if (rVar(cf->extRing)!=1)
   {
-    WerrorS("only univarite minpoly allowed");
+    WerrorS("only univariate minpoly allowed");
     return NULL;
   }
 
-  assume (r->idroot==NULL);
+  number p = n_Copy(a,cf);
+  n_Normalize(p, cf);
 
-  number p = n_Copy(a,r->cf);
-  n_Normalize(p, r->cf);
-
-  if (n_IsZero(p, r->cf))
+  if (n_IsZero(p, cf))
   {
-    n_Delete(&p, r->cf);
-    if( nCoeff_is_transExt(r->cf) )
-    {
-      return r;
-    }
-    WarnS("cannot set minpoly to 0 / alg. extension?");
-    return NULL;
+    n_Delete(&p, cf);
+    return cf;
   }
-  r=rCopy(r);
-  // remove all object currently in the ring
 
   AlgExtInfo A;
 
-  A.r = rCopy(r->cf->extRing); // Copy  ground field!
+  A.r = rCopy(cf->extRing); // Copy  ground field!
   // if minpoly was already set:
-  if( r->cf->extRing->qideal != NULL ) id_Delete(&(A.r->qideal),A.r);
+  if( cf->extRing->qideal != NULL ) id_Delete(&(A.r->qideal),A.r);
   ideal q = idInit(1,1);
   if ((p==NULL) ||(NUM((fraction)p)==NULL))
   {
     WerrorS("Could not construct the alg. extension: minpoly==0");
     // cleanup A: TODO
     rDelete( A.r );
-    rDelete(r);
     return NULL;
   }
   if (DEN((fraction)(p)) != NULL) // minpoly must be a fraction with poly numerator...!!
   {
     poly n=DEN((fraction)(p));
-    if(!p_IsConstant(n,r->cf->extRing))
+    if(!p_IsConstant(n,cf->extRing))
     {
       WarnS("denominator must be constant - ignoring it");
     }
-    p_Delete(&n,r->cf->extRing);
+    p_Delete(&n,cf->extRing);
     DEN((fraction)(p))=NULL;
   }
 
@@ -239,17 +231,12 @@ ring jjSetMinpoly(ring r, number a)
   coeffs new_cf = nInitChar(n_algExt, &A);
   if (new_cf==NULL)
   {
-    WerrorS("Could not construct the alg. extension: llegal minpoly?");
+    WerrorS("Could not construct the alg. extension: illegal minpoly?");
     // cleanup A: TODO
     rDelete( A.r );
-    rDelete(r);
     return NULL;
   }
-  else
-  {
-    nKillChar(r->cf); r->cf=new_cf;
-  }
-  return r;
+  return new_cf;
 }
 
 static BOOLEAN jjMINPOLY(leftv, leftv a)
@@ -473,16 +460,29 @@ static BOOLEAN jiA_INT(leftv res, leftv a, Subexpr e)
   }
   return FALSE;
 }
+static inline ring jjCheck_FLAG_OTHER_RING(leftv res)
+{
+  ring old_r=currRing;
+  if (Sy_inset(FLAG_RING,res->flag))
+  {
+    if ((res-1)->data!=currRing)
+    {
+      if ((res-1)->data!=NULL)
+      {
+        old_r=(ring)(res-1)->data;
+        rDecRefCnt(old_r);
+      }
+      (res-1)->data=rIncRefCnt(currRing);
+      (res-1)->rtyp=RING_CMD;
+    }
+  }
+  res->flag &= ~(Sy_bit(FLAG_OTHER_RING) |Sy_bit(FLAG_RING));
+  return old_r;
+}
 static BOOLEAN jiA_NUMBER(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   number p=(number)a->CopyD(NUMBER_CMD);
+  if (errorreported) return TRUE;
   if (res->data!=NULL) nDelete((number *)&res->data);
   nNormalize(p);
   res->data=(void *)p;
@@ -706,14 +706,8 @@ static BOOLEAN jiA_BIGINT(leftv res, leftv a, Subexpr e)
 }
 static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   syStrategy r=(syStrategy)a->CopyD(RESOLUTION_CMD);
+  if (errorreported) return TRUE;
   if (res->data!=NULL) ((lists)res->data)->Clean();
   int add_row_shift = 0;
   intvec *weights=(intvec*)atGet(a,"isHomog",INTVEC_CMD);
@@ -724,14 +718,8 @@ static BOOLEAN jiA_LIST_RES(leftv res, leftv a,Subexpr)
 }
 static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   lists l=(lists)a->CopyD(LIST_CMD);
+  if (errorreported) return TRUE;
   if (res->data!=NULL) ((lists)res->data)->Clean();
   res->data=(void *)l;
   jiAssignAttr(res,a);
@@ -739,14 +727,8 @@ static BOOLEAN jiA_LIST(leftv res, leftv a,Subexpr)
 }
 static BOOLEAN jiA_POLY(leftv res, leftv a,Subexpr e)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   poly p=(poly)a->CopyD(POLY_CMD);
+  if (errorreported) return TRUE;
   pNormalize(p);
   if (e==NULL)
   {
@@ -842,6 +824,7 @@ static BOOLEAN jiA_1x1MATRIX(leftv res, leftv a,Subexpr e)
     return TRUE;
   }
   matrix am=(matrix)a->CopyD(MATRIX_CMD);
+  if (errorreported) return TRUE;
   if ((MATROWS(am)!=1) || (MATCOLS(am)!=1))
   {
     WerrorS("must be 1x1 matrix");
@@ -932,14 +915,8 @@ static BOOLEAN jiA_BIGINTMAT(leftv res, leftv a, Subexpr)
 static BOOLEAN jiA_BUCKET(leftv res, leftv a, Subexpr e)
 // there should be no assign bucket:=bucket, here we have poly:=bucket
 {
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
   sBucket_pt b=(sBucket_pt)a->CopyD();
+  if (errorreported) return TRUE;
   poly p; int l;
   sBucketDestroyAdd(b,&p,&l);
   sleftv tmp;
@@ -950,20 +927,15 @@ static BOOLEAN jiA_BUCKET(leftv res, leftv a, Subexpr e)
 }
 static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr)
 {
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
-  void *test_p=a->Data(); // can I access it (newstruct)?
+  ideal I=(ideal)a->CopyD(MATRIX_CMD);
   if (errorreported) return TRUE;
   if (res->data!=NULL) idDelete((ideal*)&res->data);
-  res->data=(void *)a->CopyD(MATRIX_CMD);
+  res->data=(void*)I;
   if (a->rtyp==IDHDL) id_Normalize((ideal)a->Data(), currRing);
-  else                id_Normalize((ideal)res->data, currRing);
+  else                id_Normalize(I/*(ideal)res->data*/, currRing);
   jiAssignAttr(res,a);
   if (((res->rtyp==IDEAL_CMD)||(res->rtyp==MODUL_CMD))
-  && (IDELEMS((ideal)(res->data))==1)
+  && (IDELEMS(I/*(ideal)(res->data)*/)==1)
   && (currRing->qideal==NULL)
   && (!rIsPluralRing(currRing))
   )
@@ -979,33 +951,22 @@ static BOOLEAN jiA_IDEAL(leftv res, leftv a, Subexpr)
 }
 static BOOLEAN jiA_RESOLUTION(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
+  syStrategy R=(syStrategy)a->CopyD(RESOLUTION_CMD);
   if (errorreported) return TRUE;
   if (res->data!=NULL) syKillComputation((syStrategy)res->data);
-  res->data=(void *)a->CopyD(RESOLUTION_CMD);
+  res->data=(void*)R;
   jiAssignAttr(res,a);
   return FALSE;
 }
 static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr)
 /* module = poly */
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
-  if (errorreported) return TRUE;
-  if (res->data!=NULL) idDelete((ideal*)&res->data);
   ideal I=idInit(1,1);
   I->m[0]=(poly)a->CopyD(POLY_CMD);
+  if (errorreported) return TRUE;
   if (I->m[0]!=NULL) pSetCompP(I->m[0],1);
   pNormalize(I->m[0]);
+  if (res->data!=NULL) idDelete((ideal*)&res->data);
   res->data=(void *)I;
   if (TEST_V_QRING && (currRing->qideal!=NULL))
   {
@@ -1016,15 +977,8 @@ static BOOLEAN jiA_MODUL_P(leftv res, leftv a, Subexpr)
 }
 static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
-  if (res->data!=NULL) idDelete((ideal*)&res->data);
   matrix m=(matrix)a->CopyD(MATRIX_CMD);
+  if (errorreported) return TRUE;
   if (TEST_V_ALLWARN)
     if (MATROWS(m)>1)
       Warn("assign matrix with %d rows to an ideal in >>%s<<",MATROWS(m),my_yylinebuf);
@@ -1032,6 +986,7 @@ static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
   ((ideal)m)->rank=1;
   MATROWS(m)=1;
   id_Normalize((ideal)m, currRing);
+  if (res->data!=NULL) idDelete((ideal*)&res->data);
   res->data=(void *)m;
   if (TEST_V_QRING && (currRing->qideal!=NULL))
   {
@@ -1042,14 +997,8 @@ static BOOLEAN jiA_IDEAL_M(leftv res, leftv a, Subexpr)
 }
 static BOOLEAN jiA_IDEAL_Mo(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   ideal m=(ideal)a->CopyD(MODUL_CMD);
+  if (errorreported) return TRUE;
   if (m->rank>1)
   {
     Werror("rank of module is %ld in assignment to ideal",m->rank);
@@ -1093,13 +1042,6 @@ static BOOLEAN jiA_LINK(leftv res, leftv a, Subexpr)
 // assign map -> map
 static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   if (res->data!=NULL)
   {
     omFree((ADDRESS)((map)res->data)->preimage);
@@ -1107,24 +1049,19 @@ static BOOLEAN jiA_MAP(leftv res, leftv a, Subexpr)
     idDelete((ideal*)&res->data);
   }
   res->data=(void *)a->CopyD(MAP_CMD);
+  if (errorreported) return TRUE;
   jiAssignAttr(res,a);
   return FALSE;
 }
 // assign ideal -> map
 static BOOLEAN jiA_MAP_ID(leftv res, leftv a, Subexpr)
 {
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
   map f=(map)res->data;
   char *rn=f->preimage; // save the old/already assigned preimage ring name
   f->preimage=NULL;
   idDelete((ideal *)&f);
   res->data=(void *)a->CopyD(IDEAL_CMD);
+  if (errorreported) return TRUE;
   f=(map)res->data;
   id_Normalize((ideal)f, currRing);
   f->preimage = rn;
@@ -1139,18 +1076,12 @@ static BOOLEAN jiA_QRING(leftv res, leftv a,Subexpr e)
     WerrorS("qring_id expected");
     return TRUE;
   }
-  void *test_p=a->Data(); // can I access it (newstruct)?
-  if (errorreported) return TRUE;
-  if (Sy_inset(FLAG_OTHER_RING,res->flag))
-  {
-    (res-1)->data=currRing;
-    (res-1)->rtyp=RING_CMD;
-  }
 
   ring old_ring=(ring)res->Data();
 
   coeffs newcf = currRing->cf;
   ideal id = (ideal)a->Data(); //?
+  if (errorreported) return TRUE;
   const int cpos = idPosConstant(id);
   if(rField_is_Ring(currRing))
     if (cpos >= 0)
@@ -1267,7 +1198,7 @@ static BOOLEAN jiA_RING(leftv res, leftv a, Subexpr e)
       return TRUE;
     }
   }
-  r->ref++;
+  rIncRefCnt(r);
   jiAssignAttr(res,a);
   return FALSE;
 }
@@ -1285,6 +1216,7 @@ static BOOLEAN jiA_DEF(leftv res, leftv, Subexpr)
 static BOOLEAN jiA_CRING(leftv res, leftv a, Subexpr)
 {
   coeffs r=(coeffs)a->Data();
+  if (errorreported) return TRUE;
   if (r==NULL) return TRUE;
   if (res->data!=NULL) nKillChar((coeffs)res->data);
   res->data=(void *)a->CopyD(CRING_CMD);
@@ -1979,19 +1911,23 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   &&*/(ld->e==NULL)
   && (ld->Typ()!=r->Typ()))
   {
+    ring old_r=jjCheck_FLAG_OTHER_RING(ld);
     tmp.rtyp=DEF_CMD;
+    tmp.flag=ld->flag;
     b=iiAssign(&tmp,r,FALSE);
-    ld->CleanUp();
+    ld->CleanUp(old_r);
     memcpy(ld,&tmp,sizeof(sleftv));
   }
   else if ((ld->e==NULL)
   && (ld->Typ()==r->Typ())
   && (ld->Typ()<MAX_TOK))
   {
+    ring old_r=jjCheck_FLAG_OTHER_RING(ld);
     tmp.rtyp=r->Typ();
+    tmp.flag=ld->flag;
     tmp.data=(char*)idrecDataInit(r->Typ());
     b=iiAssign(&tmp,r,FALSE);
-    ld->CleanUp();
+    ld->CleanUp(old_r);
     memcpy(ld,&tmp,sizeof(sleftv));
   }
   else
@@ -2049,7 +1985,6 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
   }
   else if (l->attribute!=NULL)
     atKillAll((idhdl)l);
-  l->flag=0;
   if (ll==1)
   {
     /* l[..] = ... */
@@ -2244,7 +2179,7 @@ BOOLEAN iiAssign(leftv l, leftv r, BOOLEAN toplevel)
         return b;
       }
       //no break, handle the rest like an ideal:
-      map_assign=TRUE;
+      map_assign=TRUE; // and continue
     }
     case MATRIX_CMD:
     case IDEAL_CMD:
