@@ -413,10 +413,6 @@ lists primeFactorisation(const number n, const int pBound)
   return L;
 }
 
-#ifdef HAVE_STATIC
-#undef HAVE_DYN_RL
-#endif
-
 //#ifdef HAVE_LIBPARSER
 //#  include "libparse.h"
 //#endif /* HAVE_LIBPARSER */
@@ -806,7 +802,7 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef HAVE_FLINT
               StringAppend("FLINT(%s),",FLINT_VERSION);
 #endif
-//              StringAppendS("factory(" FACTORYVERSION "),\n\t");
+//              StringAppendS("factory(" FACTORYVERSION "),");
               StringAppendS("\n\t");
 #ifndef HAVE_OMALLOC
               StringAppendS("xalloc,");
@@ -851,7 +847,9 @@ char * versionString(/*const bool bShowDetails = false*/ )
 #ifdef HAVE_DYNAMIC_LOADING
               StringAppendS("dynamic modules,");
 #endif
-              if (p_procs_dynamic) StringAppendS("dynamic p_Procs,");
+#ifdef HAVE_DYNANIC_PPROCS
+              StringAppendS("dynamic p_Procs,");
+#endif
 #if YYDEBUG
               StringAppendS("YYDEBUG=1,");
 #endif
@@ -1477,6 +1475,8 @@ void siInit(char *name)
     SI_RESTORE_OPT(save1,save2);
   }
   // interpreter error handling
+  #ifndef __CYGWIN__
   factoryError=callWerrorS; // to honour later changes of variable WerrorS
+  #endif
   errorreported = 0;
 }
