@@ -5,19 +5,24 @@
 * ABSTRACT: computation with long rational numbers (Hubert Grassmann)
 */
 
-#include <misc/auxiliary.h>
-#include <omalloc/omalloc.h>
+#include "misc/auxiliary.h"
 
-#include <factory/factory.h>
+#include "factory/factory.h"
 
-#include <misc/sirandom.h>
-#include <misc/prime.h>
-#include <reporter/reporter.h>
+#include "misc/sirandom.h"
+#include "misc/prime.h"
+#include "reporter/reporter.h"
 
-#include "rmodulon.h" // ZnmInfo
-#include "longrat.h"
-#include "shortfl.h"
-#include "modulop.h"
+#include "coeffs/coeffs.h"
+#include "coeffs/numbers.h"
+#include "coeffs/rmodulon.h" // ZnmInfo
+#include "coeffs/longrat.h"
+#include "coeffs/shortfl.h"
+#include "coeffs/modulop.h"
+#include "coeffs/mpr_complex.h"
+
+#include <string.h>
+#include <float.h>
 
 // allow inlining only from p_Numbers.h and if ! LDEBUG
 #if defined(DO_LINLINE) && defined(P_NUMBERS_H) && !defined(LDEBUG)
@@ -57,10 +62,6 @@ BOOLEAN  nlIsMOne(number a, const coeffs r);
 long     nlInt(number &n, const coeffs r);
 number   nlBigInt(number &n);
 
-#ifdef HAVE_RINGS
-number nlMapGMP(number from, const coeffs src, const coeffs dst);
-#endif
-
 BOOLEAN  nlGreaterZero(number za, const coeffs r);
 number   nlInvers(number a, const coeffs r);
 number   nlDiv(number a, number b, const coeffs r);
@@ -71,8 +72,6 @@ void     nlPower(number x, int exp, number *lu, const coeffs r);
 const char *   nlRead (const char *s, number *a, const coeffs r);
 void     nlWrite(number a, const coeffs r);
 
-void     nlCoeffWrite(const coeffs r, BOOLEAN details);
-number   nlChineseRemainder(number *x, number *q,int rl, const coeffs C);
 number   nlFarey(number nN, number nP, const coeffs CF);
 
 #ifdef LDEBUG
@@ -110,7 +109,7 @@ BOOLEAN nlDBTest(number a, const char *f,int l, const coeffs r);
 static inline number nlShort3(number x) // assume x->s==3
 {
   assume(x->s==3);
-  if (mpz_cmp_ui(x->z,0L)==0)
+  if (mpz_sgn1(x->z)==0)
   {
     mpz_clear(x->z);
     FREE_RNUMBER(x);
@@ -133,16 +132,6 @@ static inline number nlShort3(number x) // assume x->s==3
 #ifndef LONGRAT_CC
 #define LONGRAT_CC
 
-#include <string.h>
-#include <float.h>
-
-#include <coeffs/coeffs.h>
-#include <reporter/reporter.h>
-#include <omalloc/omalloc.h>
-
-#include <coeffs/numbers.h>
-#include <coeffs/mpr_complex.h>
-
 #ifndef BYTES_PER_MP_LIMB
 #define BYTES_PER_MP_LIMB sizeof(mp_limb_t)
 #endif
@@ -153,7 +142,7 @@ static inline number nlShort3(number x) // assume x->s==3
 // #define SR_TO_INT(SR)   (((long)SR) >> 2)
 
 #define MP_SMALL 1
-//#define mpz_isNeg(A) (mpz_cmp_si(A,0L)<0)
+//#define mpz_isNeg(A) (mpz_sgn1(A)<0)
 #define mpz_isNeg(A) ((A)->_mp_size<0)
 #define mpz_limb_size(A) ((A)->_mp_size)
 #define mpz_limb_d(A) ((A)->_mp_d)
@@ -172,6 +161,17 @@ number nlShort3_noinline(number x) // assume x->s==3
   return nlShort3(x);
 }
 
+static number nlInitMPZ(mpz_t m, const coeffs)
+{
+  number z = ALLOC_RNUMBER();
+  z->s = 3;
+  #ifdef LDEBUG
+  z->debug=123456;
+  #endif
+  mpz_init_set(z->z, m);
+  z=nlShort3(z);
+  return z;
+}
 
 #if (__GNU_MP_VERSION*10+__GNU_MP_VERSION_MINOR < 31)
 void mpz_mul_si (mpz_ptr r, mpz_srcptr s, long int si)
@@ -203,17 +203,9 @@ static number nlMapR(number from, const coeffs src, const coeffs dst);
 /*2
 * convert from a GMP integer
 */
-number nlMapGMP(number from, const coeffs /*src*/, const coeffs /*dst*/)
+static inline number nlMapGMP(number from, const coeffs /*src*/, const coeffs dst)
 {
-  number z=ALLOC_RNUMBER();
-#if defined(LDEBUG)
-  z->debug=123456;
-#endif
-  mpz_init_set(z->z,(mpz_ptr) from);
-  //mpz_init_set_ui(&z->n,1);
-  z->s = 3;
-  z=nlShort3(z);
-  return z;
+  return nlInitMPZ((mpz_ptr)from,dst);
 }
 
 number nlMapZ(number from, const coeffs src, const coeffs dst)
@@ -222,7 +214,7 @@ number nlMapZ(number from, const coeffs src, const coeffs dst)
   {
     return from;
   }
-  return nlMapGMP(from,src,dst);
+  return nlInitMPZ((mpz_ptr)from,dst);
 }
 
 /*2
@@ -477,6 +469,7 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
     for (i=0;i<size;i++) dd[i] = qp[i];
     bl = 1-e;
     nn = (mp_ptr)allocfunc(sizeof(mp_limb_t)*bl);
+    memset(nn,0,sizeof(mp_limb_t)*bl);
     nn[bl-1] = 1;
     ndest = res->n;
     ndest->_mp_d = nn;
@@ -488,6 +481,109 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
     al = dest->_mp_size = size+e;
     if (al<2) al = 2;
     dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    memset(dd,0,sizeof(mp_limb_t)*al);
+    for (i=0;i<size;i++) dd[i+e] = qp[i];
+    for (i=0;i<e;i++) dd[i] = 0;
+    res->s = 3;
+  }
+
+  dest->_mp_d = dd;
+  dest->_mp_alloc = al;
+  if (negative) mpz_neg(dest,dest);
+
+  if (res->s==0)
+    nlNormalize(res,dst);
+  else if (mpz_size1(res->z)<=MP_SMALL)
+  {
+    // res is new, res->ref is 1
+    res=nlShort3(res);
+  }
+  nlTest(res, dst);
+  return res;
+}
+
+
+static number nlMapC(number from, const coeffs src, const coeffs dst)
+{
+  assume( getCoeffType(src) == n_long_C );
+  if ( ! ((gmp_complex*)from)->imag().isZero() )
+    return INT_TO_SR(0);
+
+  if (dst->is_field==FALSE) /* ->ZZ */
+  {
+    char *s=floatToStr(((gmp_complex*)from)->real(),src->float_len);
+    mpz_t z;
+    mpz_init(z);
+    char *ss=nEatLong(s,z);
+    if (*ss=='\0')
+    {
+      omFree(s);
+      number n=nlInitMPZ(z,dst);
+      mpz_clear(z);
+      return n;
+    }
+    omFree(s);
+    mpz_clear(z);
+    WarnS("conversion problem in CC -> ZZ mapping");
+    return INT_TO_SR(0);
+  }
+      
+  mpf_t *f = ((gmp_complex*)from)->real()._mpfp();
+
+  number res;
+  mpz_ptr dest,ndest;
+  int size, i,negative;
+  int e,al,bl;
+  mp_ptr qp,dd,nn;
+
+  size = (*f)[0]._mp_size;
+  if (size == 0)
+    return INT_TO_SR(0);
+  if(size<0)
+  {
+    negative = 1;
+    size = -size;
+  }
+  else
+    negative = 0;
+
+  qp = (*f)[0]._mp_d;
+  while(qp[0]==0)
+  {
+    qp++;
+    size--;
+  }
+
+  e=(*f)[0]._mp_exp-size;
+  res = ALLOC_RNUMBER();
+#if defined(LDEBUG)
+  res->debug=123456;
+#endif
+  dest = res->z;
+
+  void* (*allocfunc) (size_t);
+  mp_get_memory_functions (&allocfunc,NULL, NULL);
+  if (e<0)
+  {
+    al = dest->_mp_size = size;
+    if (al<2) al = 2;
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    for (i=0;i<size;i++) dd[i] = qp[i];
+    bl = 1-e;
+    nn = (mp_ptr)allocfunc(sizeof(mp_limb_t)*bl);
+    memset(nn,0,sizeof(mp_limb_t)*bl);
+    nn[bl-1] = 1;
+    ndest = res->n;
+    ndest->_mp_d = nn;
+    ndest->_mp_alloc = ndest->_mp_size = bl;
+    res->s = 0;
+  }
+  else
+  {
+    al = dest->_mp_size = size+e;
+    if (al<2) al = 2;
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
+    memset(dd,0,sizeof(mp_limb_t)*al);
     for (i=0;i<size;i++) dd[i+e] = qp[i];
     for (i=0;i<e;i++) dd[i] = 0;
     res->s = 3;
@@ -674,7 +770,7 @@ number nlInvers(number a, const coeffs r)
     n->s=1;
     if (((long)a)>0L)
     {
-      mpz_init_set_si(n->z,1L);
+      mpz_init_set_ui(n->z,1L);
       mpz_init_set_si(n->n,(long)SR_TO_INT(a));
     }
     else
@@ -719,7 +815,7 @@ number nlInvers(number a, const coeffs r)
               }
               else
               {
-                mpz_init_set_si(n->z,1L);
+                mpz_init_set_ui(n->z,1L);
               }
               break;
     }
@@ -753,7 +849,13 @@ number   nlExactDiv(number a, number b, const coeffs r)
     long bb=SR_TO_INT(b);
     return INT_TO_SR(aa/bb);
   }
+  number aa=NULL;
   number bb=NULL;
+  if (SR_HDL(a) & SR_INT)
+  {
+    aa=nlRInit(SR_TO_INT(a));
+    a=aa;
+  }
   if (SR_HDL(b) & SR_INT)
   {
     bb=nlRInit(SR_TO_INT(b));
@@ -766,7 +868,17 @@ number   nlExactDiv(number a, number b, const coeffs r)
   mpz_init(u->z);
   /* u=a/b */
   u->s = 3;
+  assume(a->s==3);
+  assume(b->s==3);
   mpz_divexact(u->z,a->z,b->z);
+  if (aa!=NULL)
+  {
+    mpz_clear(aa->z);
+#if defined(LDEBUG)
+    aa->debug=654324;
+#endif
+    FREE_RNUMBER(aa); // omFreeBin((void *)aa, rnumber_bin);
+  }
   if (bb!=NULL)
   {
     mpz_clear(bb->z);
@@ -950,9 +1062,10 @@ int nlDivComp(number a, number b, const coeffs r)
   return 0;
 }
 
-number  nlGetUnit (number, const coeffs)
+number  nlGetUnit (number n, const coeffs cf)
 {
-  return INT_TO_SR(1);
+  if (nlGreaterZero(n,cf)) return INT_TO_SR(1);
+  else                     return INT_TO_SR(-1);
 }
 
 coeffs nlQuot1(number c, const coeffs r)
@@ -967,13 +1080,13 @@ coeffs nlQuot1(number c, const coeffs r)
   #ifdef HAVE_RINGS
   else
   {
-    mpz_ptr dummy;
-    dummy = (mpz_ptr) omAlloc(sizeof(mpz_t));
+    mpz_t dummy;
     mpz_init_set_ui(dummy, ch);
     ZnmInfo info;
     info.base = dummy;
     info.exp = (unsigned long) 1;
     rr = nInitChar(n_Zn, (void*)&info);
+    mpz_clear(dummy);
   }
   #endif
   return(rr);
@@ -1313,7 +1426,7 @@ static int int_extgcd(int a, int b, int * u, int* x, int * v, int* y)
 number nlShort1(number x) // assume x->s==0/1
 {
   assume(x->s<2);
-  if (mpz_cmp_ui(x->z,0L)==0)
+  if (mpz_sgn1(x->z)==0)
   {
     _nlDelete_NoImm(&x);
     return INT_TO_SR(0);
@@ -1401,8 +1514,8 @@ number nlNormalizeHelper(number a, number b, const coeffs r)
   if (mpz_cmp_si(gcd,1L)!=0)
   {
     mpz_t bt;
-    mpz_init_set(bt,b->n);
-    mpz_divexact(bt,bt,gcd);
+    mpz_init(bt);
+    mpz_divexact(bt,b->n,gcd);
     if (SR_HDL(a) & SR_INT)
       mpz_mul_si(result->z,bt,SR_TO_INT(a));
     else
@@ -1464,21 +1577,21 @@ number nlModP(number q, const coeffs /*Q*/, const coeffs Zp)
 /*2
 * convert number i (from Q) to GMP and warn if denom != 1
 */
-void nlGMP(number &i, number n, const coeffs r)
+void nlGMP(number &i, mpz_t n, const coeffs r)
 {
   // Hier brauche ich einfach die GMP Zahl
   nlTest(i, r);
   nlNormalize(i, r);
   if (SR_HDL(i) & SR_INT)
   {
-    mpz_set_si((mpz_ptr) n, SR_TO_INT(i));
+    mpz_set_si(n, SR_TO_INT(i));
     return;
   }
   if (i->s!=3)
   {
      WarnS("Omitted denominator during coefficient mapping !");
   }
-  mpz_set((mpz_ptr) n, i->z);
+  mpz_set(n, i->z);
 }
 #endif
 
@@ -1565,8 +1678,8 @@ BOOLEAN _nlEqual_aNoImm_OR_bNoImm(number a, number b)
     if ((((long)a) < 0L) && (!mpz_isNeg(b->z)))
       return FALSE;
     mpz_t  bb;
-    mpz_init_set(bb,b->n);
-    mpz_mul_si(bb,bb,(long)SR_TO_INT(a));
+    mpz_init(bb);
+    mpz_mul_si(bb,b->n,(long)SR_TO_INT(a));
     bo=(mpz_cmp(bb,b->z)==0);
     mpz_clear(bb);
     return bo;
@@ -1627,6 +1740,9 @@ void _nlDelete_NoImm(number *a)
         (*a)->s=2;
 #endif
     }
+    #ifdef LDEBUG
+    memset(*a,0,sizeof(**a));
+    #endif
     FREE_RNUMBER(*a); // omFreeBin((void *) *a, rnumber_bin);
   }
 }
@@ -1691,7 +1807,7 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
         mpz_mul_si(x,b->n,SR_TO_INT(a));
         mpz_add(u->z,b->z,x);
         mpz_clear(x);
-        if (mpz_cmp_ui(u->z,0L)==0)
+        if (mpz_sgn1(u->z)==0)
         {
           mpz_clear(u->z);
           FREE_RNUMBER(u);
@@ -1740,7 +1856,7 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
             mpz_add(u->z,u->z,x);
             mpz_clear(x);
 
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -1763,7 +1879,7 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
           {
             mpz_mul(u->z,b->z,a->n);
             mpz_add(u->z,u->z,a->z);
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -1792,7 +1908,7 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
           {
             mpz_mul(u->z,a->z,b->n);
             mpz_add(u->z,u->z,b->z);
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -1986,7 +2102,7 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
         mpz_mul_si(x,b->n,SR_TO_INT(a));
         mpz_sub(u->z,x,b->z);
         mpz_clear(x);
-        if (mpz_cmp_ui(u->z,0L)==0)
+        if (mpz_sgn1(u->z)==0)
         {
           mpz_clear(u->z);
           FREE_RNUMBER(u);
@@ -2033,7 +2149,7 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
         mpz_mul_si(x,a->n,SR_TO_INT(b));
         mpz_sub(u->z,a->z,x);
         mpz_clear(x);
-        if (mpz_cmp_ui(u->z,0L)==0)
+        if (mpz_sgn1(u->z)==0)
         {
           mpz_clear(u->z);
           FREE_RNUMBER(u);
@@ -2087,7 +2203,7 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
             mpz_sub(u->z,y,x);
             mpz_clear(x);
             mpz_clear(y);
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -2113,7 +2229,7 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
             mpz_mul(x,b->z,a->n);
             mpz_sub(u->z,a->z,x);
             mpz_clear(x);
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -2145,7 +2261,7 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
             mpz_mul(x,a->z,b->n);
             mpz_sub(u->z,x,b->z);
             mpz_clear(x);
-            if (mpz_cmp_ui(u->z,0L)==0)
+            if (mpz_sgn1(u->z)==0)
             {
               mpz_clear(u->z);
               FREE_RNUMBER(u);
@@ -2308,11 +2424,33 @@ number nlCopyMap(number a, const coeffs /*src*/, const coeffs /*dst*/)
   return _nlCopy_NoImm(a);
 }
 
-nMapFunc nlSetMap(const coeffs src, const coeffs /*dst*/)
+number nlMapQtoZ(number a, const coeffs src, const coeffs dst)
+{
+  if ((SR_HDL(a) & SR_INT)||(a==NULL))
+  {
+    return a;
+  }
+  if (a->s==3) return _nlCopy_NoImm(a);
+  number a0=a;
+  BOOLEAN a1=FALSE;
+  if (a->s==0) { a0=_nlCopy_NoImm(a); a1=TRUE; }
+  number b1=nlGetNumerator(a0,src);
+  number b2=nlGetDenom(a0,src);
+  number b=nlIntDiv(b1,b2,dst);
+  nlDelete(&b1,src);
+  nlDelete(&b2,src);
+  if (a1)  _nlDelete_NoImm(&a0);
+  return b;
+}
+
+nMapFunc nlSetMap(const coeffs src, const coeffs dst)
 {
   if (src->rep==n_rep_gap_rat)  /*Q, coeffs_BIGINT */
   {
-    return ndCopyMap;
+    if ((src->is_field==dst->is_field) /* Q->Q, Z->Z*/
+    || (src->is_field==FALSE))         /* Z->Q */
+      return nlCopyMap;
+    return nlMapQtoZ;        /* Q->Z */
   }
   if ((src->rep==n_rep_int) && nCoeff_is_Zp(src))
   {
@@ -2326,8 +2464,12 @@ nMapFunc nlSetMap(const coeffs src, const coeffs /*dst*/)
   {
     return nlMapLongR; /* long R -> Q */
   }
+  if (nCoeff_is_long_C(src))
+  {
+    return nlMapC; /* C -> Q */
+  }
 #ifdef HAVE_RINGS
-  if (src->rep==n_rep_gmp) // nCoeff_is_Ring_Z(src) || nCoeff_is_Ring_PtoM(src) || nCoeff_is_Ring_ModN(src))
+  if (src->rep==n_rep_gmp) // nCoeff_is_Z(src) || nCoeff_is_Ring_PtoM(src) || nCoeff_is_Zn(src))
   {
     return nlMapGMP;
   }
@@ -2461,7 +2603,7 @@ LINLINE BOOLEAN nlIsZero (number a, const coeffs)
   }
   return FALSE;
   #else
-  return (a==INT_TO_SR(0));
+  return (a==NULL)|| (a==INT_TO_SR(0));
   #endif
 }
 
@@ -2634,7 +2776,7 @@ LINLINE void nlInpMult(number &a, number b, const coeffs r)
 
 #ifndef P_NUMBERS_H
 
-static void nlMPZ(mpz_t m, number &n, const coeffs r)
+void nlMPZ(mpz_t m, number &n, const coeffs r)
 {
   nlTest(n, r);
   nlNormalize(n, r);
@@ -2643,18 +2785,6 @@ static void nlMPZ(mpz_t m, number &n, const coeffs r)
 }
 
 
-static number nlInitMPZ(mpz_t m, const coeffs)
-{
-  number z = ALLOC_RNUMBER();
-  z->s = 3;
-  #ifdef LDEBUG
-  z->debug=123456;
-  #endif
-  mpz_init_set(z->z, m);
-  z=nlShort3(z);
-  return z;
-}
-
 number  nlXExtGcd (number a, number b, number *s, number *t, number *u, number *v, const coeffs r)
 {
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
@@ -2743,7 +2873,7 @@ number nlQuotRem (number a, number b, number * r, const coeffs R)
       *r = INT_TO_SR(rr);
     if (SR_TO_INT(b)<0)
     {
-      mpz_mul_si(qq, qq, -1);
+      mpz_neg(qq, qq);
     }
     return nlInitMPZ(qq,R);
   }
@@ -2795,21 +2925,23 @@ void nlInpIntDiv(number &a, number b, const coeffs r)
 
 number nlFarey(number nN, number nP, const coeffs r)
 {
-  mpz_t tmp; mpz_init(tmp);
-  mpz_t A,B,C,D,E,N,P;
-  if (SR_HDL(nN) & SR_INT) mpz_init_set_si(N,SR_TO_INT(nN));
-  else                     mpz_init_set(N,nN->z);
+  mpz_t A,B,C,D,E,N,P,tmp;
   if (SR_HDL(nP) & SR_INT) mpz_init_set_si(P,SR_TO_INT(nP));
   else                     mpz_init_set(P,nP->z);
+  const mp_bitcnt_t bits=2*(mpz_size1(P)+1)*GMP_LIMB_BITS;
+  mpz_init2(N,bits);
+  if (SR_HDL(nN) & SR_INT) mpz_set_si(N,SR_TO_INT(nN));
+  else                     mpz_set(N,nN->z);
   assume(!mpz_isNeg(P));
   if (mpz_isNeg(N))  mpz_add(N,N,P);
-  mpz_init_set_si(A,0L);
-  mpz_init_set_ui(B,(unsigned long)1);
-  mpz_init_set_si(C,0L);
-  mpz_init(D);
-  mpz_init_set(E,P);
+  mpz_init2(A,bits); mpz_set_ui(A,0L);
+  mpz_init2(B,bits); mpz_set_ui(B,1L);
+  mpz_init2(C,bits); mpz_set_ui(C,0L);
+  mpz_init2(D,bits);
+  mpz_init2(E,bits); mpz_set(E,P);
+  mpz_init2(tmp,bits);
   number z=INT_TO_SR(0);
-  while(mpz_cmp_si(N,0L)!=0)
+  while(mpz_sgn1(N)!=0)
   {
     mpz_mul(tmp,N,N);
     mpz_add(tmp,tmp,tmp);
@@ -2829,8 +2961,8 @@ number nlFarey(number nN, number nP, const coeffs r)
          #ifdef LDEBUG
          z->debug=123456;
          #endif
-         mpz_init_set(z->z,N);
-         mpz_init_set(z->n,B);
+         memcpy(z->z,N,sizeof(mpz_t));
+         memcpy(z->n,B,sizeof(mpz_t));
          z->s = 0;
          nlNormalize(z,r);
        }
@@ -2838,6 +2970,8 @@ number nlFarey(number nN, number nP, const coeffs r)
        {
          // return nN (the input) instead of "fail"
          z=nlCopy(nN,r);
+         mpz_clear(B);
+         mpz_clear(N);
        }
        break;
     }
@@ -2853,11 +2987,9 @@ number nlFarey(number nN, number nP, const coeffs r)
   }
   mpz_clear(tmp);
   mpz_clear(A);
-  mpz_clear(B);
   mpz_clear(C);
   mpz_clear(D);
   mpz_clear(E);
-  mpz_clear(N);
   mpz_clear(P);
   return z;
 }
@@ -2911,15 +3043,13 @@ number nlExtGcd(number a, number b, number *s, number *t, const coeffs)
   return g;
 }
 
-void    nlCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
-{
-  if (r->is_field)
-  PrintS("//   characteristic : 0\n");
-  else
-  PrintS("//   coeff. ring is : Integers\n");
-}
+//void    nlCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
+//{
+//  if (r->is_field)  PrintS("QQ");
+//  else  PrintS("ZZ");
+//}
 
-int n_SwitchChinRem=0;
+VAR int n_SwitchChinRem=0;
 number   nlChineseRemainderSym(number *x, number *q,int rl, BOOLEAN sym, CFArray &inv_cache,const coeffs CF)
 // elemenst in the array are x[0..(rl-1)], q[0..(rl-1)]
 {
@@ -2941,24 +3071,28 @@ number   nlChineseRemainderSym(number *x, number *q,int rl, BOOLEAN sym, CFArray
   if (sym)
   {
     number p=CF->convFactoryNSingN(qnew,CF);
-    number p2=nlIntDiv(p,nlInit(2, CF),CF);
-    if (nlGreater(n,p2,CF))
+    number p2;
+    if (getCoeffType(CF) == n_Q) p2=nlIntDiv(p,nlInit(2, CF),CF);
+    else                         p2=CF->cfDiv(p,CF->cfInit(2, CF),CF);
+    if (CF->cfGreater(n,p2,CF))
     {
-       number n2=nlSub(n,p,CF);
-       nlDelete(&n,CF);
+       number n2=CF->cfSub(n,p,CF);
+       CF->cfDelete(&n,CF);
        n=n2;
     }
-    nlDelete(&p2,CF);
-    nlDelete(&p,CF);
+    CF->cfDelete(&p2,CF);
+    CF->cfDelete(&p,CF);
   }
-  nlNormalize(n,CF);
+  CF->cfNormalize(n,CF);
   return n;
 }
-number   nlChineseRemainder(number *x, number *q,int rl, const coeffs C)
+#if 0
+number nlChineseRemainder(number *x, number *q,int rl, const coeffs C)
 {
   CFArray inv(rl);
   return nlChineseRemainderSym(x,q,rl,TRUE,inv,C);
 }
+#endif
 
 static void nlClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
 {
@@ -3151,33 +3285,26 @@ char * nlCoeffName(const coeffs r)
   else                 return (char*)"ZZ";
 }
 
-static char* nlCoeffString(const coeffs r)
-{
-  //return omStrDup(nlCoeffName(r));
-  if (r->cfDiv==nlDiv) return omStrDup("QQ");
-  else                 return omStrDup("ZZ");
-}
-
-static void nlWriteFd(number n,FILE* f, const coeffs)
+void nlWriteFd(number n, const ssiInfo* d, const coeffs)
 {
   if(SR_HDL(n) & SR_INT)
   {
     #if SIZEOF_LONG == 4
-    fprintf(f,"4 %ld ",SR_TO_INT(n));
+    fprintf(d->f_write,"4 %ld ",SR_TO_INT(n));
     #else
     long nn=SR_TO_INT(n);
     if ((nn<POW_2_28_32)&&(nn>= -POW_2_28_32))
     {
       int nnn=(int)nn;
-      fprintf(f,"4 %d ",nnn);
+      fprintf(d->f_write,"4 %d ",nnn);
     }
     else
     {
       mpz_t tmp;
       mpz_init_set_si(tmp,nn);
-      fputs("8 ",f);
-      mpz_out_str (f,SSI_BASE, tmp);
-      fputc(' ',f);
+      fputs("8 ",d->f_write);
+      mpz_out_str (d->f_write,SSI_BASE, tmp);
+      fputc(' ',d->f_write);
       mpz_clear(tmp);
     }
     #endif
@@ -3185,29 +3312,29 @@ static void nlWriteFd(number n,FILE* f, const coeffs)
   else if (n->s<2)
   {
     //gmp_fprintf(f,"%d %Zd %Zd ",n->s,n->z,n->n);
-    fprintf(f,"%d ",n->s+5);
-    mpz_out_str (f,SSI_BASE, n->z);
-    fputc(' ',f);
-    mpz_out_str (f,SSI_BASE, n->n);
-    fputc(' ',f);
+    fprintf(d->f_write,"%d ",n->s+5);
+    mpz_out_str (d->f_write,SSI_BASE, n->z);
+    fputc(' ',d->f_write);
+    mpz_out_str (d->f_write,SSI_BASE, n->n);
+    fputc(' ',d->f_write);
 
     //if (d->f_debug!=NULL) gmp_fprintf(d->f_debug,"number: s=%d gmp/gmp \"%Zd %Zd\" ",n->s,n->z,n->n);
   }
   else /*n->s==3*/
   {
     //gmp_fprintf(d->f_write,"3 %Zd ",n->z);
-    fputs("8 ",f);
-    mpz_out_str (f,SSI_BASE, n->z);
-    fputc(' ',f);
+    fputs("8 ",d->f_write);
+    mpz_out_str (d->f_write,SSI_BASE, n->z);
+    fputc(' ',d->f_write);
 
     //if (d->f_debug!=NULL) gmp_fprintf(d->f_debug,"number: gmp \"%Zd\" ",n->z);
   }
 }
 
-static number nlReadFd(s_buff f, const coeffs)
+number nlReadFd(const ssiInfo *d, const coeffs)
 {
   int sub_type=-1;
-  sub_type=s_readint(f);
+  sub_type=s_readint(d->f_read);
   switch(sub_type)
   {
      case 0:
@@ -3215,8 +3342,8 @@ static number nlReadFd(s_buff f, const coeffs)
        {// read mpz_t, mpz_t
          number n=nlRInit(0);
          mpz_init(n->n);
-         s_readmpz(f,n->z);
-         s_readmpz(f,n->n);
+         s_readmpz(d->f_read,n->z);
+         s_readmpz(d->f_read,n->n);
          n->s=sub_type;
          return n;
        }
@@ -3224,7 +3351,7 @@ static number nlReadFd(s_buff f, const coeffs)
      case 3:
        {// read mpz_t
          number n=nlRInit(0);
-         s_readmpz(f,n->z);
+         s_readmpz(d->f_read,n->z);
          n->s=3; /*sub_type*/
          #if SIZEOF_LONG == 8
          n=nlShort3(n);
@@ -3233,7 +3360,7 @@ static number nlReadFd(s_buff f, const coeffs)
        }
      case 4:
        {
-         LONG dd=s_readlong(f);
+         LONG dd=s_readlong(d->f_read);
          //#if SIZEOF_LONG == 8
          return INT_TO_SR(dd);
          //#else
@@ -3245,15 +3372,15 @@ static number nlReadFd(s_buff f, const coeffs)
        {// read raw mpz_t, mpz_t
          number n=nlRInit(0);
          mpz_init(n->n);
-         s_readmpz_base (f,n->z, SSI_BASE);
-         s_readmpz_base (f,n->n, SSI_BASE);
+         s_readmpz_base (d->f_read,n->z, SSI_BASE);
+         s_readmpz_base (d->f_read,n->n, SSI_BASE);
          n->s=sub_type-5;
          return n;
        }
      case 8:
        {// read raw mpz_t
          number n=nlRInit(0);
-         s_readmpz_base (f,n->z, SSI_BASE);
+         s_readmpz_base (d->f_read,n->z, SSI_BASE);
          n->s=sub_type=3; /*subtype-5*/
          #if SIZEOF_LONG == 8
          n=nlShort3(n);
@@ -3266,6 +3393,7 @@ static number nlReadFd(s_buff f, const coeffs)
   }
   return NULL;
 }
+
 BOOLEAN nlCoeffIsEqual(const coeffs r, n_coeffType n, void *p)
 {
   /* test, if r is an instance of nInitCoeffs(n,parameter) */
@@ -3307,11 +3435,9 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   r->is_domain=TRUE;
   r->rep=n_rep_gap_rat;
 
-  //const int ch = (int)(long)(p);
-
   r->nCoeffIsEqual=nlCoeffIsEqual;
   //r->cfKillChar = ndKillChar; /* dummy */
-  r->cfCoeffString=nlCoeffString;
+  //r->cfCoeffString=nlCoeffString;
   r->cfCoeffName=nlCoeffName;
 
   r->cfInitMPZ = nlInitMPZ;
@@ -3320,6 +3446,7 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   r->cfMult  = nlMult;
   r->cfSub   = nlSub;
   r->cfAdd   = nlAdd;
+  r->cfExactDiv= nlExactDiv;
   if (p==NULL) /* Q */
   {
     r->is_field=TRUE;
@@ -3342,7 +3469,6 @@ BOOLEAN nlInitChar(coeffs r, void*p)
     r->cfXExtGcd=nlXExtGcd;
     r->cfQuotRem=nlQuotRem;
   }
-  r->cfExactDiv= nlExactDiv;
   r->cfInit = nlInit;
   r->cfSize  = nlSize;
   r->cfInt  = nlInt;
@@ -3373,7 +3499,7 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   //r->cfName = ndName;
   r->cfInpMult=nlInpMult;
   r->cfInpAdd=nlInpAdd;
-  r->cfCoeffWrite=nlCoeffWrite;
+  //r->cfCoeffWrite=nlCoeffWrite;
 
   r->cfClearContent = nlClearContent;
   r->cfClearDenominators = nlClearDenominators;
@@ -3391,8 +3517,6 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   r->cfWriteFd=nlWriteFd;
   r->cfReadFd=nlReadFd;
 
-  // the variables: general stuff (required)
-  r->nNULL = INT_TO_SR(0);
   //r->type = n_Q;
   r->ch = 0;
   r->has_simple_Alloc=FALSE;
