@@ -118,7 +118,12 @@ static number ndInvers(number a, const coeffs r)
   return res;
 }
 
-static BOOLEAN ndIsUnit(number a, const coeffs r) { return !r->cfIsZero(a,r); }
+static BOOLEAN ndIsUnit_Ring(number a, const coeffs r)
+{ return r->cfIsOne(a,r)|| r->cfIsMOne(a,r); }
+static BOOLEAN ndIsUnit_Field(number a, const coeffs r)
+{ return !r->cfIsZero(a,r); }
+static number ndGetUnit_Ring(number a, const coeffs r)
+{ return r->cfInit(1,r); }
 #ifdef LDEBUG
 // static void   nDBDummy1(number* d,char *, int) { *d=NULL; }
 static BOOLEAN ndDBTest(number, const char *, const int, const coeffs){ return TRUE; }
@@ -142,8 +147,7 @@ static int ndParDeg(number n, const coeffs r)
 
 static number ndParameter(const int, const coeffs r)
 {
-  Werror("ndParameter: n_Parameter is not implemented/relevant for (coeff_type = %d)",getCoeffType(r));
-  return NULL;
+  return r->cfInit(1,r);
 }
 
 BOOLEAN n_IsZeroDivisor( number a, const coeffs r)
@@ -160,7 +164,7 @@ BOOLEAN n_IsZeroDivisor( number a, const coeffs r)
   return ret;
 }
 
-static void   ndNormalize(number&, const coeffs) { }
+void   ndNormalize(number&, const coeffs) { }
 static number ndReturn0(number, const coeffs r)        { return r->cfInit(0,r); }
 number ndGcd(number, number, const coeffs r)    { return r->cfInit(1,r); }
 static number ndIntMod(number, number, const coeffs r) { return r->cfInit(0,r); }
@@ -404,12 +408,12 @@ coeffs nInitChar(n_coeffType t, void * parameter)
     n->cfClearContent = ndClearContent;
     n->cfClearDenominators = ndClearDenominators;
 
-    n->cfIsUnit = ndIsUnit;
+    //n->cfIsUnit = ndIsUnit;
 #ifdef HAVE_RINGS
     n->cfDivComp = ndDivComp;
     n->cfDivBy = ndDivBy;
     n->cfExtGcd = ndExtGcd;
-    //n->cfGetUnit = (nMapFunc)NULL;
+    //n->cfGetUnit = ndGetUnit;
 #endif
 
 #ifdef LDEBUG
@@ -435,10 +439,18 @@ coeffs nInitChar(n_coeffType t, void * parameter)
     if (n->cfRePart==NULL) n->cfRePart=n->cfCopy;
     if (n->cfExactDiv==NULL) n->cfExactDiv=n->cfDiv;
     if (n->cfSubringGcd==NULL) n->cfSubringGcd=n->cfGcd;
-
-#ifdef HAVE_RINGS
-    if (n->cfGetUnit==NULL) n->cfGetUnit=n->cfCopy;
-#endif
+    if (n->cfIsUnit==NULL)
+    {
+      if (n->is_field) n->cfIsUnit=ndIsUnit_Field;
+      else             n->cfIsUnit=ndIsUnit_Ring;
+    }
+    #ifdef HAVE_RING
+    if (n->cfGetUnit==NULL)
+    {
+      if (n->is_field) n->cfGetUnit=n->cfCopy;
+      else             n->cfGetUnit=ndGetUnit_Ring;
+    }
+    #endif
 
     if(n->cfWriteShort==NULL)
       n->cfWriteShort = n->cfWriteLong;
