@@ -8,11 +8,11 @@
 
 /* #define SHIFT_MULT_DEBUG */
 
-/* enable compat mode until the user interface is updated to support xy instead of x(1)*y(2)
- * NOTE: it already works, but all tests and the libraries need to be updated first
- * -> wait until the new interface is released
-*/
-#define SHIFT_MULT_COMPAT_MODE // ncHilb.lib still relies on it
+/*
+ * NEEDED BY
+ * - ncHilb.lib
+ */
+#define SHIFT_MULT_COMPAT_MODE 
 
 #ifdef SHIFT_MULT_DEBUG
 #include "../kernel/polys.h"
