@@ -5,23 +5,21 @@
 
 set -e
 
-TARVERSION=4.2.0
+TARVERSION=4.2.0p3
 VERSION=4.2.0
-BUILD_DIR=/tmp2/wawa
+BUILD_DIR=/tmp/tst2
 export VERSION TARVERSION BUILD_DIR
 
 # sanity check
 if test -e $BUILD_DIR/singularconfig.h
 then
 
+rm -rf singular-$VERSION
 git archive --prefix=singular-$VERSION/ HEAD |tar xf -
 mkdir singular-$VERSION/doc
-cp doc/*.man singular-$VERSION/doc/.
-cp doc/singular.idx singular-$VERSION/doc/.
-if test -e doc/doc.tbz2
-then
-  cp doc/doc.tbz2 singular-$VERSION/doc/.
-fi
+cp doc/*.* singular-$VERSION/doc/.
+mkdir singular-$VERSION/doc/images
+cp doc/images/* singular-$VERSION/doc/images/.
 
 command rm singular-$VERSION/.gdbinit singular-$VERSION/*/.gdbinit singular-$VERSION/*/*/.gdbinit singular-$VERSION/*/*/*/.gdbinit
 command rm singular-$VERSION/IntegerProgramming/README
